/*
 * nrf_ptx_m328p.c
 *
 * Created: 2014-11-28 16:07:37
 *  Author: michal
 */ 


#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include "spi.h"
#include "hal_nrf_reg.h"
#include "nrf24l01p.h"
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/sfr_defs.h>
#include <avr/wdt.h>
#include "ds18b20.h"
#include <avr/power.h>


#define RF_PAYLOAD_LENGTH	1
#define RF_CHANNEL 82

#define POWERUP_SENSOR() PORTD |= (1<<PD4);
#define POWERDOWN_SENSOR() PORTD &= ~(1<<PD4);

#define DHT_PORT_OUT PORTD
#define DHT_PORT_IN PIND
#define DHT_PIN PD6

uint8_t address_tx[]={10,10,0,0,5};	//LSB byte first, TX_ADDR in PTX, RX_ADDR_0 in PRX

uint8_t databuffer[5]={55,1,0x91, 1, 55};	//max size one payload in nrf24l01p
	
volatile uint8_t sekundy = 58 ;

void Init(void);
void nrf_Init(void);
uint8_t fetchData();


int main(void) {	

		 wdt_enable	(WDTO_8S);
		 wdt_reset();
		 DDRC |= (1<<PC1) /*| (1<<PC5)*/;	//out
		  		PORTC |= (1<<PC1);	//high
		  		_delay_ms(500);
		  		PORTC &= ~(1<<PC1);	//low
		Init();

		 		

	sei();			
    while(1) {	
			
			wdt_reset();
 					
			 set_sleep_mode(SLEEP_MODE_IDLE);
 			sleep_mode();
			 
			if(sekundy == 59)
			{
			DDRD &= ~( 1<<PD5 | 1<< PD6);
			POWERUP_SENSOR();
			}

			if(sekundy == 60)
				{
					cli();
					ds18b20_gettemp();
					fetchData();
					sekundy = 0;
					nrf_write_multibyte_reg(HAL_NRF_TX_PLOAD,databuffer,5,0);	
					//trigger TX
					
					POWERDOWN_SENSOR();
					DDRD |= (1<<PD4 | 1<<PD5 | 1<< PD6); // as output
					PORTD &= ~(1<<PD4 | 1<<PD5 | 1<< PD6); // set low - power down sensors
					
					PORTB |= 1<<PD1;	// CE=high, trigger TX mode
					_delay_us(15);	//wait >10us
					PORTB &= ~(1<<PD1);	// CE=low, end trgiger TX mode
					sei();
				}

			}
}


void Init(void){	
		

	SPI_MasterInit();
	nrf_Init();
	
	PRR |= (1<<PRTWI | 1<< PRTIM2 | 1<<PRUSART0 | 1<<PRADC);
	
	//Timer1
	OCR1A = 976;
	TCCR1B = 1<<WGM12 | 1<<CS10 | 1<<CS12;
	TIMSK1 = 1<<OCIE1A;
	
	//blink led

	
	// IRQ PCINT0
	//PORTB |= 1<<PB0;	//pull-up IRQ
	PCICR |= (1<<PCIE0);
	PCMSK0 |= (1<<PCINT0);
	
	// Sensor Power
 	DDRD |= (1<<PD4);
 	POWERUP_SENSOR();
	 _delay_ms(2300);
	 ds18b20_gettemp();
 	 fetchData();
 	// PORTC |= (1<<PC1);	//high
}

void nrf_Init(void){
	_delay_ms(100);	//Power on reset=100ms (little more, depends on circuit and voltage regulator)
	
	DDRB |= 1<<DDD1;	//CE is output
	
	

	nrf_write_register(STATUS, 0xFF);	//clear interrupt flags
	nrf_write_register(FLUSH_TX,0);	//remove any data
	nrf_write_register(FLUSH_RX,0);	//remove any data
	
	nrf_write_register(EN_RXADDR, 0);	//disable all pipes
	nrf_write_register(EN_AA, 0);	//disable all ACK pipes
	nrf_write_register(EN_RXADDR, 1<<HAL_NRF_PIPE0);	//enable data pipe 0
	nrf_write_register(EN_AA, 1<<HAL_NRF_PIPE0);		//enable ACK data pipe 0
	nrf_write_register(CONFIG, (nrf_read_register(CONFIG) & ~(1<<CRCO)) | 1<<EN_CRC);	//enable 1 byte CRC
	//nrf_write_register(CONFIG, nrf_read_register(CONFIG) | 1<<CRCO | 1<<EN_CRC);	//enable 2 byte CRC
	
	nrf_write_register(SETUP_RETR, ARD_1500US | ARC_3);	//auto retransmit delay=1500us (250kbps), auto retransmit count=3
	nrf_write_register(SETUP_AW, 0x01);	//address field width = 3 bytes (see size address_Xx tables)
	nrf_write_multibyte_reg(HAL_NRF_TX, address_tx, 0, 0);	//set device address
	nrf_write_multibyte_reg(HAL_NRF_PIPE0, address_tx, 0, 0);	//set receiving address on pipe0
	nrf_write_register(FEATURE, nrf_read_register(FEATURE) | 1<<EN_ACK_PAY);	//try enable payload with ACK
	// When the features are locked, the FEATURE and DYNPD are read out 0x00
	// even after we have tried to enable ack payload. This mean that we need to
	// activate the features.
	if(nrf_read_register(FEATURE) == 0x00 && (nrf_read_register(DYNPD) == 0x00)){	//TODO: looks little weird
		nrf_lock_unlock();
		nrf_write_register(FEATURE, nrf_read_register(FEATURE) | 1<<EN_ACK_PAY);	//enable payload with ACK
	}
	nrf_write_register(FEATURE, nrf_read_register(FEATURE) | 1<<EN_DPL);	//enable dynamic payload length
	nrf_write_register(DYNPD, 0x3F);	//enable dynamic payload length for all data pipes
	
	//depends from device mode PRX or PTX
	//nrf_write_register(CONFIG, nrf_read_register(CONFIG) | 1<<PRIM_RX);	//for PRX device PRIM_RX=1
	nrf_write_register(CONFIG, nrf_read_register(CONFIG) & ~(1<<PRIM_RX));	//for PTX device PRIM_RX=0
	//nrf_write_register(RX_PW_P0, RF_PAYLOAD_LENGTH);	//byte payload length ---- only for PRX device
	
	nrf_write_register(RF_CH, RF_CHANNEL);
	nrf_write_register(RF_SETUP, 1<<RF_DR_LOW | HAL_NRF_0DBM<<1);	//data rate=250kbps, power=0dBm
	nrf_write_register(CONFIG, nrf_read_register(CONFIG) | 1<<PWR_UP);	//power up device
	_delay_ms(5);	//wait Tpd2stby

	
}

void SPI_MasterInit(void){
	/* Set MOSI,SCK and SS-CSN output, all others input */
	DDRB |= (1<<PB3)|(1<<PB5)|(1<<PB2);
	PORTB |= 1<<PB1 | 1<<PB4;	//pull up SS, MISO
	/* Enable SPI, Master, set clock rate fck/16 */
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
}

char SPI_MasterTransmit(char cData){
	/* Start transmission */
	SPDR = cData;
	/* Wait for transmission complete */
	while(!(SPSR & (1<<SPIF)))
		;	
	return SPDR;
}

ISR(PCINT0_vect){	
	
	//triggered by IRQ pin from nrf24l01p
	if(bit_is_clear(PINB, 0)){

		//PORTC |= (1<<PC1);	//high
	
	switch ((nrf_write_register(STATUS, 1<<MAX_RT | 1<<TX_DS | 1<<RX_DR)) & (1<<MAX_RT | 1<<TX_DS | 1<<RX_DR))	//read status register and clear interrupt flags
	{
		case (1<<HAL_NRF_MAX_RT):					// Max retries reached
			nrf_write_register(FLUSH_TX, 0);	// flush tx fifo, avoid fifo jam
			//uart_putc(0x01);
		break;
			
		case (1<<HAL_NRF_TX_DS):                  // Packet sent, and ACK received
			//nrf_write_register(FLUSH_TX,0);		//ignore temporary ACK payload
			//uart_putc(0x02);
		break;
			
		case (1<<HAL_NRF_RX_DR):                  // Packet received
		while (! ((nrf_write_register(NOP,0) & 0x0E) == 0x0E))	//while RF FIFO is not Empty RX_P_NO != 111
			nrf_read_multibyte_reg(HAL_NRF_RX_PLOAD, databuffer);	//read payload data
			//uart_putc(0x03);
		break;
			
		case ((1<<HAL_NRF_RX_DR)|(1<<HAL_NRF_TX_DS)): // Ack payload recieved
		while (! ((nrf_write_register(NOP,0) & 0x0E) == 0x0E))	//while RF FIFO is not Empty RX_P_NO != 111
			nrf_read_multibyte_reg(HAL_NRF_RX_PLOAD, databuffer);	//read payload ACK data
			//uart_putc(0x04);
		break;
			
		default:
			//uart_putc(0x05);
		break;
	}
	} 
	//else PORTC &= ~(1<<PC1);	//low
	
}

ISR(TIMER1_COMPA_vect)
{
	sekundy++;
}

uint8_t fetchData()
{
    uint8_t data [5];
    uint8_t cnt, check;
    int8_t i,j;
    
    /******************* Sensor communication start *******************/
    
    /* Set data pin as output first */
	
    DDRD |= _BV(DHT_PIN);
    
    
    /* Clear bit for 20 ms */
    DHT_PORT_OUT &= ~_BV(DHT_PIN);
    /* Wait about 20 ms */
    _delay_ms(30);
    
    /* Now set Timer0 with clk/8 prescaling.
     Gives 1s per cycle @8Mhz */
    TCCR0B = 0x01;
    
    TCNT0 = 0;
    
    /* Pull high again */
    DHT_PORT_OUT |= _BV(DHT_PIN);
    
    /* And set data pin as input */
    DDRD &= ~_BV(DHT_PIN);
    
    /* Wait for response from sensor, 20-40s according to datasheet */
     while(bit_is_set(DHT_PORT_IN, DHT_PIN))
 	
     { if (TCNT0 >= 60) return 0; }
   
    /************************* Sensor preamble *************************/
    
    TCNT0 = 0;
    
    /* Now wait for the first response to finish, low ~80s */
    while(bit_is_clear(DHT_PORT_IN, DHT_PIN))
    { if (TCNT0 >= 100) return 0; }
    
    TCNT0 = 0;
    
    /* Then wait for the second response to finish, high ~80s */
    while(bit_is_set(DHT_PORT_IN, DHT_PIN))
    { if (TCNT0 >= 100) return 0; }
    
    /********************* Data transmission start **********************/
    
    for (i = 0; i < 5; ++i)
    {
        for(j = 7; j >= 0; --j)
        {
            TCNT0 = 0;
            
            /* First there is always a 50s low period */
            while(bit_is_clear(DHT_PORT_IN, DHT_PIN))
            { if (TCNT0 >= 70) return 0; }
            
            TCNT0 = 0;
            
            /* Then the data signal is sent. 26 to 28s (ideally)
             indicate a low bit, and around 70s a high bit */
            while(bit_is_set(DHT_PORT_IN, DHT_PIN))
            { if (TCNT0 >= 90) return 0; }
            
            /* Store the value now so that the whole checking doesn't
             move the TCNT0 forward by too much to make the data look
             bad */
			
            cnt = TCNT0;
			
            if (cnt >= 10 && cnt <= 55)
            { data[i] &= ~_BV(j); }
            
            else if (cnt >= 56 && cnt <= 90)
            { data[i] |= _BV(j); }
            
            else return 0;
			
        }
    }
    
    /********************* Sensor communication end *********************/
    TCCR0B = 0x0;
    check = (data[0] + data[1] + data[2] + data[3]) & 0xFF;
   	
   		
    if (check != data[4]) return 0;
//   // DHT_PORT_OUT &= ~_BV(DHT_PIN);
//    // for(i = 0; i < 4; ++i)
//    // { arr[i] = data[i]; }
    databuffer[3] = data[0];
    return 1;
}